@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2017 Zabbix SIA
+** Copyright (C) 2001-2018 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -59,7 +59,10 @@ $availableJScripts = [
 	'servercheck.js' => '',
 	'flickerfreescreen.js' => '',
 	'multiselect.js' => '',
+	'colorpicker.js' => '',
 	'chkbxrange.js' => '',
+	'csvggraphwidget.js' => '',
+	'layout.mode.js' => '',
 	// vendors
 	'prototype.js' => 'vendors/',
 	'jquery.js' => 'vendors/',
@@ -72,37 +75,33 @@ $availableJScripts = [
 	'class.cmap.js' => '',
 	'class.cmessages.js' => '',
 	'class.cookie.js' => '',
+	'class.coverride.js' => '',
+	'class.crangecontrol.js' => '',
 	'class.cscreen.js' => '',
 	'class.csuggest.js' => '',
-	'class.cswitcher.js' => '',
+	'class.csvggraph.js' => '',
 	'class.ctree.js' => '',
 	'class.curl.js' => '',
-	'class.rpc.js' => '',
+	'class.cverticalaccordion.js' => '',
 	'class.svg.canvas.js' => 'vector/',
 	'class.svg.map.js' => 'vector/',
-	'class.pmaster.js' => '',
 	'class.cviewswitcher.js' => '',
+	'class.pmaster.js' => '',
+	'class.rpc.js' => '',
 	'init.js' => '',
 	// templates
 	'sysmap.tpl.js' => 'templates/',
 	// page-specific scripts
 	'items.js' => 'pages/',
-	'tr_logform.js' => 'pages/',
 ];
 
 $tranStrings = [
 	'gtlc.js' => [
-		'S_ALL_S' => _('All'),
-		'S_ZOOM' => _('Zoom'),
-		'S_FIXED_SMALL' => _('fixed'),
-		'S_DYNAMIC_SMALL' => _('dynamic'),
-		'S_NOW_SMALL' => _('now'),
 		'S_YEAR_SHORT' => _x('y', 'year short'),
 		'S_MONTH_SHORT' => _x('m', 'month short'),
 		'S_DAY_SHORT' => _x('d', 'day short'),
 		'S_HOUR_SHORT' => _x('h', 'hour short'),
-		'S_MINUTE_SHORT' => _x('m', 'minute short'),
-		'S_DATE_FORMAT' => DATE_TIME_FORMAT
+		'S_MINUTE_SHORT' => _x('m', 'minute short')
 	],
 	'dashboard.grid.js' => [
 		'Edit widget' => _('Edit widget'),
@@ -114,17 +113,20 @@ $tranStrings = [
 		'Delete' => _('Delete'),
 		'You have unsaved changes.' => _('You have unsaved changes.'),
 		'Are you sure, you want to leave this page?' => _('Are you sure, you want to leave this page?'),
-		'Add a new widget' => _('Add a new widget')
+		'Cannot add widgets in kiosk mode' => _('Cannot add widgets in kiosk mode'),
+		'Add a new widget' => _('Add a new widget'),
+		'Adjust widget refresh interval' => _('Adjust widget refresh interval'),
+		'%1$s, selected' => _x('%1$s, selected', 'screen reader'),
+		'Displaying %1$s of %2$s found' => _('Displaying %1$s of %2$s found')
 	],
 	'functions.js' => [
 		'Cancel' => _('Cancel'),
+		'Close' => _('Close'),
 		'Execute' => _('Execute'),
 		'Execution confirmation' => _('Execution confirmation')
 	],
-	'tr_logform.js' => [
-		'S_REMOVE' => _('Remove')
-	],
 	'class.calendar.js' => [
+		'S_Calendar' => _('Calendar'),
 		'S_JANUARY' => _('January'),
 		'S_FEBRUARY' => _('February'),
 		'S_MARCH' => _('March'),
@@ -137,16 +139,20 @@ $tranStrings = [
 		'S_OCTOBER' => _('October'),
 		'S_NOVEMBER' => _('November'),
 		'S_DECEMBER' => _('December'),
+		'S_MONDAY' => _('Monday'),
+		'S_TUESDAY' => _('Tuesday'),
+		'S_WEDNESDAY' => _('Wednesday'),
+		'S_THURSDAY' => _('Thursday'),
+		'S_FRIDAY' => _('Friday'),
+		'S_SATURDAY' => _('Saturday'),
+		'S_SUNDAY' => _('Sunday'),
 		'S_MONDAY_SHORT_BIG' => _x('M', 'Monday short'),
 		'S_TUESDAY_SHORT_BIG' => _x('T', 'Tuesday short'),
 		'S_WEDNESDAY_SHORT_BIG' => _x('W', 'Wednesday short'),
 		'S_THURSDAY_SHORT_BIG' => _x('T', 'Thursday short'),
 		'S_FRIDAY_SHORT_BIG' => _x('F', 'Friday short'),
 		'S_SATURDAY_SHORT_BIG' => _x('S', 'Saturday short'),
-		'S_SUNDAY_SHORT_BIG' => _x('S', 'Sunday short'),
-		'S_NOW' => _('Now'),
-		'S_DONE' => _('Done'),
-		'S_TIME' => _('Time')
+		'S_SUNDAY_SHORT_BIG' => _x('S', 'Sunday short')
 	],
 	'class.cmap.js' => [
 		'S_ON' => _('On'),
@@ -161,7 +167,7 @@ $tranStrings = [
 		'S_DEFAULT' => _('Default'),
 		'S_PLEASE_SELECT_TWO_ELEMENTS' => _('Please select two elements'),
 		'S_DOT' => _('Dot'),
-		'S_TWO_ELEMENTS_SHOULD_BE_SELECTED' => _('Two elements should be selected'),
+		'S_TWO_MAP_ELEMENTS_SHOULD_BE_SELECTED' => _('Two map elements should be selected'),
 		'S_DELETE_SELECTED_ELEMENTS_Q' => _('Delete selected elements?'),
 		'S_DELETE_SELECTED_SHAPES_Q' => _('Delete selected shapes?'),
 		'S_BRING_TO_FRONT' => _('Bring to front'),
@@ -176,6 +182,7 @@ $tranStrings = [
 		'S_INCORRECT_ELEMENT_MAP_LINK' => _('All links should have "Name" and "URL" specified'),
 		'S_EACH_URL_SHOULD_HAVE_UNIQUE' => _('Each URL should have a unique name. Please make sure there is only one URL named'),
 		'S_DELETE_LINKS_BETWEEN_SELECTED_ELEMENTS_Q' => _('Delete links between selected elements?'),
+		'S_MACRO_EXPAND_ERROR' => _('Cannot expand macros.'),
 		'S_NO_IMAGES' => 'You need to have at least one image uploaded to create map element. Images can be uploaded in Administration->General->Images section.',
 		'Colour "%1$s" is not correct: expecting hexadecimal colour code (6 symbols).' => _('Colour "%1$s" is not correct: expecting hexadecimal colour code (6 symbols).')
 	],
@@ -189,16 +196,32 @@ $tranStrings = [
 	'class.cookie.js' => [
 		'S_MAX_COOKIE_SIZE_REACHED' => _('We are sorry, the maximum possible number of elements to remember has been reached.')
 	],
-	'main.js' => [
-		'S_EXPAND' => _('Expand'),
+	'class.coverride.js' => [
+		'S_COLOUR' => _('colour'),
+		'S_TIME_SHIFT' => _('time shift')
+	],
+	'class.cverticalaccordion.js' => [
 		'S_COLLAPSE' => _('Collapse'),
+		'S_EXPAND' => _('Expand')
+	],
+	'main.js' => [
+		'Close' => _('Close')
 	],
 	'multiselect.js' => [
 		'No matches found' => _('No matches found'),
 		'More matches found...' => _('More matches found...'),
 		'type here to search' => _('type here to search'),
 		'new' => _('new'),
-		'Select' => _('Select')
+		'Select' => _('Select'),
+		'Added, %1$s' => _x('Added, %1$s', 'screen reader'),
+		'Removed, %1$s' => _x('Removed, %1$s', 'screen reader'),
+		'%1$s, read only' => _x('%1$s, read only', 'screen reader'),
+		'Can not be removed' => _x('Can not be removed', 'screen reader'),
+		'Selected, %1$s in position %2$d of %3$d' => _x('Selected, %1$s in position %2$d of %3$d', 'screen reader'),
+		'Selected, %1$s, read only, in position %2$d of %3$d' => _x('Selected, %1$s, read only, in position %2$d of %3$d', 'screen reader'),
+		'More than %1$d matches for %2$s found' => _x('More than %1$d matches for %2$s found', 'screen reader'),
+		'%1$d matches for %2$s found' => _x('%1$d matches for %2$s found', 'screen reader'),
+		'%1$s preselected, use down,up arrow keys and enter to select' => _x('%1$s preselected, use down,up arrow keys and enter to select', 'screen reader')
 	],
 	'menupopup.js' => [
 		'Acknowledge' => _('Acknowledge'),
@@ -208,6 +231,7 @@ $tranStrings = [
 		'Create trigger' => _('Create trigger'),
 		'Dashboard sharing' => _('Dashboard sharing'),
 		'Delete service "%1$s"?' => _('Delete service "%1$s"?'),
+		'Description' => _('Description'),
 		'Do you wish to replace the conditional expression?' => _('Do you wish to replace the conditional expression?'),
 		'Edit trigger' => _('Edit trigger'),
 		'Insert expression' => _('Insert expression'),
@@ -225,13 +249,11 @@ $tranStrings = [
 		'Last month graph' => _('Last month graph'),
 		'Last week graph' => _('Last week graph'),
 		'Problems' => _('Problems'),
-		'Refresh time' => _('Refresh time'),
-		'Refresh time multiplier' => _('Refresh time multiplier'),
+		'Refresh interval' => _('Refresh interval'),
+		'Refresh interval multiplier' => _('Refresh interval multiplier'),
 		'Scripts' => _('Scripts'),
-		'Something went wrong. Please try again later!' => _('Something went wrong. Please try again later!'),
 		'Submap' => _('Submap'),
 		'Trigger' => _('Trigger'),
-		'Triggers' => _('Triggers'),
 		'Update' => _('Update'),
 		'URL' => _('URL'),
 		'URLs' => _('URLs'),
@@ -258,6 +280,9 @@ $tranStrings = [
 		'Cancel' => _('Cancel'),
 		'Add child element' => _('Add child element'),
 		'Add multiple maps' => _('Add multiple maps')
+	],
+	'colorpicker.js' => [
+		'Close' => _('Close')
 	]
 ];
 
@@ -305,11 +330,18 @@ foreach ($files as $file) {
 	}
 }
 
-$jsLength = strlen($js);
-$etag = md5($jsLength);
-if (isset($_SERVER['HTTP_IF_NONE_MATCH']) && $_SERVER['HTTP_IF_NONE_MATCH'] == $etag) {
+$etag = md5($js);
+/**
+ * strpos function allow to check ETag value to fix cases when web server compression is used:
+ * - For case when apache server appends "-gzip" suffix to ETag.
+ *   https://bz.apache.org/bugzilla/show_bug.cgi?id=39727
+ *   https://bz.apache.org/bugzilla/show_bug.cgi?id=45023
+ * - For case when nginx v1.7.3+ server mark ETag as weak adding "W/" prefix
+ *   http://nginx.org/en/CHANGES
+ */
+if (array_key_exists('HTTP_IF_NONE_MATCH', $_SERVER) && strpos($_SERVER['HTTP_IF_NONE_MATCH'], $etag) !== false) {
 	header('HTTP/1.1 304 Not Modified');
-	header('ETag: '.$etag);
+	header('ETag: "'.$etag.'"');
 	exit;
 }
 
@@ -327,10 +359,10 @@ if (in_array('prototype.js', $files)) {
 		'};';
 }
 
-header('Content-type: text/javascript; charset=UTF-8');
+header('Content-Type: application/javascript; charset=UTF-8');
 // breaks if "zlib.output_compression = On"
 // header('Content-length: '.$jsLength);
 header('Cache-Control: public, must-revalidate');
-header('ETag: '.$etag);
+header('ETag: "'.$etag.'"');
 
 echo $js;
